#include <boost/interprocess/shared_memory_object.hpp>
#include <boost/interprocess/mapped_region.hpp>
#include <cstring>
#include <cstdlib>
#include <string>
#include <iostream>

#define MAX_MSG_LENGTH  50
#define TYPES   8

struct msg_s {
	int type;
	char content[MAX_MSG_LENGTH];
};

int main(int argc, char *argv[])
{
   
	using namespace boost::interprocess;
	if(argc == 1){  //Parent process
		//Remove shared memory on construction and destruction
		struct shm_remove 
		{
			shm_remove() { shared_memory_object::remove("MySharedMemory"); }
			~shm_remove(){ std::cout<<"Parent: SM removed"<<std::endl; shared_memory_object::remove("MySharedMemory"); }
		} remover;

		//Create a shared memory object.
		shared_memory_object shm (create_only, "MySharedMemory", read_write);
		std::cout<<"Parent: SM created"<<std::endl;
		//Set size
		shm.truncate(sizeof(msg_s));

		//Map the whole shared memory in this process
		mapped_region region(shm, read_write);
		msg_s* shared_msg = (struct msg_s *)region.get_address();
		
		srandom(time(NULL));
		/* producing a message on the shared segment */
		shared_msg->type = random() % TYPES;
		strcpy(shared_msg->content, "Hello!!!");
		region.flush();
		std::cout<<"Parent: My message, type: " << shared_msg->type << " & content: " << shared_msg->content << std::endl;

		//Launch child process
		std::string s(argv[0]); s += " child ";
		if(0 != std::system(s.c_str()))
			return 1;
	}
	else {
		//Open already created shared memory object.
		shared_memory_object shm (open_only, "/example", read_only);

		//Map the whole shared memory in this process
		mapped_region region(shm, read_only);
		msg_s* shared_msg = (struct msg_s *)region.get_address();
	
		std::cout<<"Child: Message, type: " << shared_msg->type << " & content: " << shared_msg->content << std::endl;	
		
	}
	return 0;
}

