#include <boost/interprocess/shared_memory_object.hpp>
#include <boost/interprocess/mapped_region.hpp>
#include <boost/interprocess/offset_ptr.hpp>
#include <iostream>

struct structure
{
	int integer1;   //The compiler places this at offset 0 in the structure
	boost::interprocess::offset_ptr<int> ptr;        //The compiler places this at offset 4 in the structure
	int * ptr2;   //The compiler places this at offset 8 in the structure	
};

int main(int argc, char *argv[])
{
	using namespace boost::interprocess;
	if(argc == 1){  //Parent process
		//Remove shared memory on construction and destruction
		struct shm_remove 
		{
			shm_remove() { shared_memory_object::remove("MySharedMemory"); }
			~shm_remove(){ shared_memory_object::remove("MySharedMemory"); }
		} remover;

		shared_memory_object shm (create_only, "MySharedMemory", read_write);
		
		shm.truncate(sizeof(structure));

		mapped_region region(shm, read_write);
		structure* s = (struct structure *)region.get_address();
		s->integer1 = 1234;		
		
		std::cout<<"Parent:"<<std::endl;
		std::cout<<&(s->integer1)<<std::endl;
		s->ptr2 = &(s->integer1);
		s->ptr = &(s->integer1);
		std::cout<<*(s->ptr2)<<std::endl;
		std::cout<<*(s->ptr)<<std::endl;
		
		region.flush();
		std::string child(argv[0]); child += " child ";
		if(0 != std::system(child.c_str()))
			return 1;
	}
	else {
		shared_memory_object shm (open_only, "MySharedMemory", read_only);

		mapped_region region(shm, read_only);
		structure* s = (struct structure *)region.get_address();
		std::cout<<"Child:"<<std::endl;		
		std::cout<<&(s->integer1)<<std::endl;
		std::cout<<*(s->ptr)<<std::endl;
		std::cout<<*(s->ptr2)<<std::endl;		
				
	}

	structure s;
	s.ptr = s.ptr++;
	
	//ptr is equal to null
	assert(!s.ptr);

	//This is the same as assigning the null value...
	s.ptr = 0;

	//ptr is also equal to null
	assert(!s.ptr);	

	return 0;
}
